import unittest
import pyminerva as mi



def test():
    mi.analyse_techs()



'''
Main Fuction
'''

if __name__ == "__main__":

    test('SPY')








# from pyminerva import PyMinerva

# class Test(unittest.TestCase):
#     def test_lower_method(self):
#         self.assertEqual(PyMinerva.lower("TEST"), "test")
#         self.assertNotEqual(PyMinerva.lower("test"), "TEST")

#     def test_upper_method(self):
#         self.assertEqual(PyMinerva.upper("test"), "TEST")
#         self.assertNotEqual(PyMinerva.upper("TEST"), "test")

#     def test_title_method(self):
#         # self.assertEqual(PyMinerva.title("hello world"), "Hello world")
#         self.assertNotEqual(PyMinerva.title("hELLO wORLD"), "hello world")


# Copyright 2023-2025 Jeongmin Kang, jarvisNim @ GitHub
# See LICENSE for details.

import numpy as np

def daily_returns(prices):
    res = (prices/prices.shift(1) - 1.0)[1:]
    res.columns = ['return']
    return res

def cumulative_returns(returns):
    res = (returns + 1.0).cumprod()
    res.columns = ['cumulative return']
    return res

def max_drawdown(cum_returns):
    max_returns = np.fmax.accumulate(cum_returns)
    res = cum_returns / max_returns - 1
    res.columns = ['max drawdown']
    return res




import pandas as pd

from zipline.data.bundles import register
from zipline.data.bundles.csvdir import csvdir_equities

start_session = pd.Timestamp('2017-01-01', tz='utc')
end_session = pd.Timestamp('2019-06-30', tz='utc')

# register the bundle 
register(
    'quandl', # name we select for the bundle
    csvdir_equities(
        ['Data'], # name of the directory as specified above (named after data frequency)
        './codes/jupyter/Minerva/Data/AAPL.csv', # path to directory containing the data
    ),
    calendar_name='NYSE',  # US equities
    start_session=start_session,
    end_session=end_session
)


# Copyright 2023-2025 Jeongmin Kang, jarvisNim @ GitHub
# See LICENSE for details.

__author__ = "jarvisNim in GitHub"
__version__ = "0.0.5"


from .techs import (
    analyse_techs,
    analyse_DrawDown,
)



# Copyright 2023-2025 Jeongmin Kang, jarvisNim @ GitHub
# See LICENSE for details.

import pandas as pd
import matplotlib.pyplot as plt
import yfinance as yf

from .utils import constant as cst
from .data.techs_data import (
    daily_returns,
    cumulative_returns,
    max_drawdown,
)

'''
공통 영역
'''
ta_list = ['mfi', 'ppo', 'bb', 'sma', 'rsi', 'vwap']



'''
funtions
'''

def analyse_techs(ticker=None):
    """
    This function retrieves all the available technical Analysis indexed on pandas_ta. 
    This function also allows the users to specify which technical anlysis do they want to draw graphic;
    so on, all the technical graphs will be returned.

    Args:
        ticker (:obj:`str`, optional): name of the ticker to analyse all its available technical analysis from.

    Returns:
        :obj:`pandas.DataFrame` - analysis:

                country | name | full_name | symbol | isin | asset_class | currency | stock_exchange | def_stock_exchange
                --------|------|-----------|--------|------|-------------|----------|----------------|--------------------
                xxxxxxx | xxxx | xxxxxxxxx | xxxxxx | xxxx | xxxxxxxxxxx | xxxxxxxx | xxxxxxxxxxxxxx | xxxxxxxxxxxxxxxxxx

    Raises:
        ValueError: raised when any of the input arguments is not valid.
        FileNotFoundError: raised when `etfs.csv` file was not found.
        IOError: raised when `etfs.csv` file is missing.

    """

    return None


def analyse_DrawDown(tickers:list):
    """
    Max DrawDown 그래프 생성
    Args:
        tickers(:obj:'list', optional) yahoo finance 기준 list of tickers
    Returns:
        :obj: 'file' - drawdown_{tickers}.png
    Raises:
        ValueError: raised when any of the input arguments is not valid.      
    """

    if tickers is not None and not isinstance(tickers, list): # input
        raise ValueError("ERR#0025: specified tickers value not valid.")


    if ddown is None: # output
        raise ValueError("ERR#0025: ddown not found or unable to retrieve.")
    
    
    threshold_value = -0.3
    plt.figure(figsize=(16,4*len(tickers)))
    for i, tick in enumerate(tickers):
        ticker = yf.Ticker(tick)
        prices = ticker.history(period='12y')['Close'] # 12: life cycle
        dret = daily_returns(prices)
        cret = cumulative_returns(dret)
        ddown = max_drawdown(cret)
        ddown[ddown.values < -0.3]

        plt.subplot(len(tickers), 1, i + 1)
        plt.grid()
        plt.bar(ddown.index, ddown, color='royalblue')
        plt.title(ticker)
        plt.axhline(y=threshold_value, color='red', linestyle='--', label='Threshold')
        plt.xlabel('Date')
        plt.ylabel('Draw Down %')

    plt.tight_layout()  # 서브플롯 간 간격 조절
    plt.show()
    # plt.savefig(f'drawdown_{tickers}.png')

    return ddown 









# https://codingandfun.com/rsi-momentum-strategies-using-python/
import requests
import pandas as pd
import numpy as np

apiKey = 'f57bdcaa7d140c9de35806d47fbd2f91'
stock = 'TSM'
stockPrices = requests.get(f'https://financialmodelingprep.com/api/v3/historical-price-full/{stock}?apikey={apiKey}').json()
stockPrices = stockPrices['historical'][0:1200]
stockPrices = pd.DataFrame(stockPrices)
stockPrices = stockPrices.set_index('date')
stockPrices = stockPrices.iloc[::-1]

stockPrices['return'] = np.log(stockPrices['close'] / stockPrices['close'].shift(1))
stockPrices['movement'] = stockPrices['close'] - stockPrices['close'].shift(1)

stockPrices['up'] = np.where((stockPrices['movement'] > 0), stockPrices['movement'], 0)
stockPrices['down'] = np.where((stockPrices['movement'] < 0), stockPrices['movement'], 0)

window_length = 14
up = stockPrices['up'].rolling(window_length).mean()
down = stockPrices['down'].abs().rolling(window_length).mean()

RS = up / down

RSI = 100.0 - (100.0 / (1.0 + RS))
RSI = RSI.rename('RSI')

# print(RSI)

new = pd.merge(stockPrices, RSI, left_index=True, right_index=True)
new['long'] = np.where((new['RSI'] < 30), 1, np.nan)
new['long'] = np.where((new['RSI'] > 70), 0, new['long'])
new['long'].ffill(inplace=True)
new['gain_loss'] = new['long'].shift(1) * new['return']
new['total'] = new['gain_loss'].cumsum()
print(new.head())
print(new.tail())


import feedparser
import pandas as pd
import datetime as dt
from datetime import datetime

# List of RSS feed URLs
rss_feed_urls = [
    "https://www.oecd-ilibrary.org/rss/content/collection/mei-data-en/latest?fmt=rss",
]

print('--------------')
print(datetime.now())

def fetch_rss_data(url):
    feed = feedparser.parse(url)
    print("Feed Title:", feed.feed.title)
    for entry in feed.entries:
        if entry.title == 'Consumer prices':
            start_time = dt.datetime(2020, 1, 1)
            end_time = dt.datetime(2022, 2, 1)
            print("Entry Title:", entry.title)
            print("Entry Link:", entry.link)
            print("Entry Published Date:", entry.published)
            print("Entry Summary:", entry.summary)
            print("\n")
            df = web.DataReader('PRICES_CPI', 'oecd', start_time, end_time)
    else:
        continue


# Fetch data from multiple RSS feeds
for url in rss_feed_urls:
    fetch_rss_data(url)